# -*- coding: utf-8 -*-

import logging
import functools
import time

logger = logging.getLogger('SmarTool')


# 参考py38新语法, "/"前的参数严格遵守位置, 且不会提示出来
def retry(func=None, /,  times=5, delay=2, catch_error=None, ignore_error=None):

    times = int(times)
    delay = float(delay)
    catch_error = _check_error_parameter(catch_error)
    ignore_error = _check_error_parameter(ignore_error)

    if catch_error and ignore_error:
        logger.warning("Both catch error and ignore error provided, only use catch error!")
        ignore_error = []

    def wrapper(f):
        @functools.wraps(f)
        def inner(*args, **kwargs):
            count = 0
            while 1:
                try:
                    return f(*args, **kwargs)
                except Exception as e:
                    # 如果传入了需要retry的catch_error且实际发生的error不属于其中，则直接raise
                    if catch_error and not _check_raised_error(e, catch_error):
                        raise
                    # 如果传入了不需要retry的ignore_error且实际发生的error属于其中，则直接raise
                    if ignore_error and _check_raised_error(e, ignore_error):
                        raise

                    count += 1
                    logger.warning(f'Failed to run function "{f.__name__}", retry {times-count}, error: {e}')
                    if count == times:
                        raise
                    time.sleep(delay)
        return inner

    if func is None:
        return wrapper
    elif callable(func):
        return wrapper(func)
    else:
        raise ValueError(f'Invalid parameter: {func} is not callable')


def _check_error_parameter(errors):
    if errors is None:
        return []
    elif isinstance(errors, (list, tuple)):
        for error in errors:
            _check_error_parameter(error)
        return list(errors)
    elif issubclass(errors, Exception):
        return [errors]
    else:
        raise ValueError(f'Invalid parameter "{errors}", must be Exception/List[Exception]/Tuple[Exception]')


def _check_raised_error(error, error_list):
    for e in error_list:
        if isinstance(error, e):
            return True
    return False



import logging

from .datetime import DTUtil
from .retry import retry
from .timeout import TimeoutException, timeout

logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s: %(message)s')


def hello():
    welcome_msg = "===== Hello, this is SmarTool! ====="
    print(welcome_msg)
    print(" Author: Harpsichord")
    print(" Email: tliu1217@163.com")
    print(" Install: pip install SmarTool")
    print(" Github: https://github.com/Harpsichord1207/SmarTool")
    print("=" * len(welcome_msg))


__all__ = [retry.__name__, hello.__name__, timeout.__name__, TimeoutException.__name__]



# -*- coding: utf-8 -*-
import functools
import signal


class TimeoutException(Exception):

    def __init__(self, func_name, timeout_):
        self.func_name = func_name
        self.timeout = timeout_

    def __str__(self):
        return f'Function {self.func_name} timeout after {self.timeout}s'


def timeout(func=None, /,  seconds=5):

    seconds = int(seconds)

    def wrapper(f):

        def handler(signum, frame):
            raise TimeoutException(f.__name__, seconds)

        @functools.wraps(f)
        def inner(*args, **kwargs):
            signal.signal(signal.SIGALRM, handler)
            signal.alarm(seconds)
            try:
                return f(*args, **kwargs)
            except Exception:
                raise
            finally:
                signal.signal(signal.SIGALRM, signal.SIG_IGN)

        return inner

    if func is None:
        return wrapper
    elif callable(func):
        return wrapper(func)
    else:
        raise ValueError(f'Invalid parameter: {func} is not callable')



import pathlib
from setuptools import setup

read_me = pathlib.Path(__file__).parent.joinpath('README.md').read_text()

setup_args = {
    'name': 'SmarTool',
    'version': '1.0.6',
    'description': 'A set of tools that keep Python sweeter.',
    'long_description': read_me,
    'long_description_content_type': 'text/markdown',
    'author': 'Harpsichord',
    'author_email': 'tliu1217@163.com',
    'license': 'MPL 2.0',
    'packages': ['SmarTool'],
    'package_dir': {'SmarTool': 'src'},
    'url': 'https://github.com/Harpsichord1207/SmarTool',
    'install_requires': [],
    'classifiers': [
        'Programming Language :: Python :: 3.8',
        'License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)',
        'Intended Audience :: Developers'
    ],
    'keywords': 'Python Util Tool Convenience',
    'platforms': 'any',
    'zip_safe': True
}

setup(**setup_args)
